/*-
 * Copyright (c) 2004
 *     Michael B. Durian
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 4. The name of the Author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
#ifndef INTREPIDCS_H
#define INTREPIDCS_H

#define ICS_NETID_DEVICE	0x00
#define ICS_NETID_HSCAN		0x01
#define ICS_NETID_MSCAN		0x02
#define ICS_NETID_SWCAN		0x03
#define ICS_NETID_LSFTCAN	0x04
#define ICS_NETID_FORDSCP	0x05
#define ICS_NETID_J1708		0x06
#define ICS_NETID_AUX		0x07
#define ICS_NETID_JVPW		0x08
#define ICS_NETID_ISO		0x09
#define ICS_NETID_ISOPIC	0x0a
#define ICS_NETID_MAIN51	0x0b
#define ICS_NETID_HOST		0x0c


#define ICS_MSG_TYPE_DEV_STATUS		0x01
#define ICS_MSG_TYPE_DEV_RX_OVFL	0x02
#define ICS_MSG_TYPE_DEV_TX_COMPL	0x03
#define ICS_MSG_TYPE_DEV_CAN_ERROR	0x04
#define ICS_MSG_TYPE_DEV_PIC_OVFL	0x05

#define ICS_MSG_TYPE_MAIN51_TX_OVFL	0x03

#define ICS_MAX_DATA_SIZE 15
#define ICS_MAX_CAN_DATA 8

/* packet defintion */
struct ics_packet {
	uint8_t	target;
	uint8_t	len;
	uint8_t data[ICS_MAX_DATA_SIZE];
};

/* can message defintion */
struct ics_can_msg {
	uint8_t		netid;
	uint16_t	timestamp;
	uint32_t	can_id;
	int		is_extended_id;
	int		is_remote_frame;
	uint8_t		data[ICS_MAX_CAN_DATA];
	uint8_t		data_len;
};

/* opaque data types */
struct ics_interface;
struct ics_stream_state;

/* data structures used in stream callbacks */
struct ics_dev_status {
	uint8_t	sync_count;
	uint8_t	reset_status;
};

struct ics_rx_overflow {
	uint8_t	netid;
	uint16_t	timestamp;
};

struct ics_tx_complete {
	uint8_t	netid;
	uint16_t	timestamp;
	uint16_t	desc;
};

struct ics_can_error {
	uint8_t	netid;
	uint16_t	timestamp;

	/* error flags */
	uint8_t	rx1ovr : 1;
	uint8_t	rx0ovr : 1;
	uint8_t	txbo : 1;
	uint8_t	txep : 1;
	uint8_t	rxep : 1;
	uint8_t	txwar : 1;
	uint8_t	rxwar : 1;
	uint8_t	ewarn : 1;
};

struct ics_main51_overflow {
	uint8_t	netid;
};

/* callback function type definitions */
typedef void (ics_device_status_func)(struct ics_stream_state *ss,
    struct ics_dev_status *arg, void *client_data);
typedef void (ics_rx_overflow_func)(struct ics_stream_state *ss,
    struct ics_rx_overflow *arg, void *client_data);
typedef void (ics_tx_complete_func)(struct ics_stream_state *ss,
    struct ics_tx_complete *arg, void *client_data);
typedef void (ics_can_error_func)(struct ics_stream_state *ss,
    struct ics_can_error *arg, void *client_data);
typedef void (ics_pic_overflow_func)(struct ics_stream_state *ss,
    void *client_data);
typedef void (ics_main51_overflow_func)(struct ics_stream_state *ss,
    struct ics_main51_overflow *arg, void *client_data);
typedef void (ics_hscan_func)(struct ics_stream_state *ss,
    struct ics_can_msg *arg, void *client_data);
typedef void (ics_swcan_func)(struct ics_stream_state *ss,
    struct ics_can_msg *arg, void *client_data);
typedef void (ics_mscan_func)(struct ics_stream_state *ss,
    struct ics_can_msg *arg, void *client_data);
typedef void (ics_lsftcan_func)(struct ics_stream_state *ss,
    struct ics_can_msg *arg, void *client_data);

/*
 * Functions used to control and communicate with the Intrepid
 * vehicle bus interfaces.
 */
struct ics_interface *ics_interface_init(const char *port, char *err,
    int err_len);
void ics_interface_release(struct ics_interface *iface);
int ics_interface_fd(struct ics_interface *iface);
const char *ics_interface_err_str(struct ics_interface *iface);
int ics_interface_write_packet(struct ics_interface *iface,
    struct ics_packet *p);
int ics_interface_read_packet(struct ics_interface *iface,
    struct ics_packet *p, int *is_valid);

/* Functions used in managing packets */
void ics_packet_format(char *str, int str_len, struct ics_packet *p);
int ics_packet_unpack_can_msg(struct ics_packet *p, struct ics_can_msg *msg);

void ics_packet_create_can_enable(struct ics_packet *p);
void ics_packet_create_can_disable(struct ics_packet *p);
void ics_packet_create_can_msg(struct ics_packet *p, struct ics_can_msg *msg);

/* Function used to process a stream of packets */
struct ics_stream_state *ics_stream_init(struct ics_interface *iface,
    char *err, int err_len);
void ics_stream_release(struct ics_stream_state *ss);
void *ics_stream_add_filter(struct ics_stream_state *ss, uint32_t mask,
    uint32_t id);
void ics_stream_remove_filter(struct ics_stream_state *ss, void *filter_id);
int ics_stream_process_packet(struct ics_stream_state *ss,
    struct ics_packet *p);

void ics_stream_set_device_status_callback(struct ics_stream_state *ss,
    ics_device_status_func *func, void *client_data);
void ics_stream_set_rx_overflow_callback(struct ics_stream_state *ss,
    ics_rx_overflow_func *func, void *client_data);
void ics_stream_set_tx_complete_callback(struct ics_stream_state *ss,
    ics_tx_complete_func *func, void *client_data);
void ics_stream_set_can_error_callback(struct ics_stream_state *ss,
    ics_can_error_func *func, void *client_data);
void ics_stream_set_pic_overflow_callback(struct ics_stream_state *ss,
    ics_pic_overflow_func *func, void *client_data);
void ics_stream_set_main51_overflow_callback(struct ics_stream_state *ss,
    ics_main51_overflow_func *func, void *client_data);
void ics_stream_set_hscan_callback(struct ics_stream_state *ss,
    ics_hscan_func *func, void *client_data);
void ics_stream_set_swcan_callback(struct ics_stream_state *ss,
    ics_swcan_func *func, void *client_data);
void ics_stream_set_mscan_callback(struct ics_stream_state *ss,
    ics_mscan_func *func, void *client_data);
void ics_stream_set_lsftscan_callback(struct ics_stream_state *ss,
    ics_lsftcan_func *func, void *client_data);

/* stream helper functions (callbacks might want to use these) */
double ics_stream_get_timestamp_tick(struct ics_stream_state *ss);
double ics_stream_get_timestamp_overflow(struct ics_stream_state *ss);
void ics_stream_update_device_status(struct ics_stream_state *ss,
    struct ics_dev_status *ds);
void ics_stream_format_can_msg(struct ics_stream_state *ss, char *msg_text,
    int max_text, struct ics_can_msg *msg, int delta);

#endif /* INTREPIDCS_H */
